/**************************************************************/
/*                                                            */
/*  Program to help convert a CICS MAP to move into a new     */
/*   commarea.  The program will increase the levels of       */
/*   the COBOL statements, so the the map itself beomes a     */
/*   level 2.                                                 */
/*                                                            */
/*  Written by Jim MacNair - October, 1997                    */
/*                                                            */
/**************************************************************/

/**************************************************************/
/* Required include for C functions                           */
/**************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

/**************************************************************/
/*                                                            */
/* Defines and global variables                               */
/*                                                            */
/**************************************************************/

#define MAX_LINE_LEN  80
#define NEWMAP_FILE_EXT ""

void printHelp()

{
   printf("\n   Command usage is:\n\n");
   printf("     CHGMAP input-file <output-file>\n\n");
   printf(" If no output file name is specified, "); 
   printf("the converted program\nwill have a file ");
   printf("extension of .NEW appended automatically.\n");
}

int main(int argc, char *argv[])

{
   FILE        *mapfile;     /* input COBOL program         */
   FILE        *newmap;      /* converted COBOL program     */
   FILE        *msgfile;     /* conversion program messages */
   int         inCounter=0;  /* input line counter          */
   int         outCounter=0; /* output line counter         */
   int         i;            /* working variable            */
   char        *fileExt;
   char mapFileName[255];
   char newFileName[255];
   char msgFileName[255];
   char mapLine[MAX_LINE_LEN + 1];


/**************************************************************/
/*     Parse the file name and generate new file name.        */
/**************************************************************/

   if (argc >= 2)        /* check for file name */
   {
      strcpy(mapFileName,argv[1]);
   }
   else
   {
      printHelp();
      exit(1);
   } /* endif */


/**************************************************************/
/* Generate the new file name                                 */
/**************************************************************/

   /* look for a period */
   strcpy(newFileName,mapFileName);
   fileExt = strchr(newFileName, '.');
   if (fileExt != NULL)
   {
      /* remove the file extension if it exists */
      (*fileExt) = 0;
   } /* endif */

   strcpy(msgFileName,newFileName);
   strcat(newFileName,".new");
   strcat(msgFileName,".msg");

/**************************************************************/
/* Check if a new file name was specified.                    */
/**************************************************************/

   if (argc > 2)	/* check for second parameter */
   {
	   strcpy(newFileName, argv[2]);
   }

/**************************************************************/
/* Open the cobol program, new file and message file.         */
/**************************************************************/

   if ((mapfile = fopen(mapFileName, "r")) == NULL)
   {
      exit(100);
   }

   if ((newmap = fopen(newFileName, "w")) == NULL)
   {
      fclose(mapfile);
      exit(100);
   }

   if ((msgfile = fopen(msgFileName, "w")) == NULL)
   {
      fclose(mapfile);
      fclose(newmap);
      exit(100);
   }

/**************************************************************/
/*                                                            */
/* Routine to build the new map                               */
/*                                                            */
/*  This routine will read the map file and generate a        */
/*  new map.  If the first two characters of a line are       */
/*  numberic, then the number will be increased by one.       */
/*                                                            */
/**************************************************************/

   while (1)
   {
      /* read the next line in the source file */
      if (fgets(mapLine, MAX_LINE_LEN, mapfile) == NULL)
      {
         break;
      }

      /* increment the line counter */
      inCounter++;

      /* check for a comment */
      if ((strlen(mapLine) > 6) && (mapLine[6] != '*'))
      {
         /* start at column 8 */
         i=7;
         /* scan for the first non-blank */
         while ((mapLine[i] == ' ') && ((size_t) i < strlen(mapLine)))
         {
            i++;
         } /* endwhile */

         /* Check if we found one before the last char */
         if ((size_t) i < (strlen(mapLine) - 1))
         {
            if ((mapLine[i] >= '0') && (mapLine[i] <= '9') &&
                (mapLine[i + 1] >= '0') && (mapLine[i + 1] <= '9'))
            {
               if (mapLine[i + 1] == '9')
               {
                  mapLine[i + 1] = '0';
                  mapLine[i]++;
               }
               else
               {
                  mapLine[i + 1]++;
               } /* endif */
            } /* endif */
         } /* endif */
      } /* endif procedure division */

      /* write line to new map file */
      fprintf(newmap,"%s",mapLine);

      /* increment the line counter for the output program */
      outCounter++;
   } /* endwhil */

   fprintf(msgfile,"\nInput lines read %5d\n",inCounter);
   fprintf(msgfile,"Output lines written %5d\n",outCounter);

   fclose(mapfile);
   fclose(newmap);
   fclose(msgfile);

/**************************************************************/
/* Finished - return normally                                 */
/**************************************************************/

   return 0;

}

/*-------------*/
/* End Of File */
/*-------------*/
