      ****************************************************************
      *    SAMPLE PROGRAM TO VERIFY USER ID AND PASSWORD, USING      *
      *     THE CICS EXEC CICS VERIFY PASSWORD API.  IT ALSO WILL    *
      *     ASSIGN A FOUR CHARACTER CLIENT NAME TO THE CLIENT.       *
      *     THIS NAME COULD BE USED BY THE APPLICATION AS A FOUR     *
      *     CHARACTER TERMINAL ID.                                   *
      *                                                              *
      *    WRITTEN BY JIM MACNAIR                                    *
      *                                                              *
      *                                                              *
      *                                                              *
      ****************************************************************
       IDENTIFICATION DIVISION.
       PROGRAM-ID. VERIFYPW.
       ENVIRONMENT DIVISION.
       DATA DIVISION.
       WORKING-STORAGE SECTION.
      *
      * RETURN CODES FROM EXEC CICS STATEMENTS
      *
       77  CICSRESP  PIC S9(8) COMP VALUE ZERO.
       77  CICSRESP2 PIC S9(8) COMP VALUE ZERO.
      *
      * WORK VARIABLE FOR LAST DATE AND TIME LAST SIGNED ON
      *
       77  LASTUSED PIC S9(15) COMP-3 VALUE ZEROS.
      *
      * WORK VARIABLES FOR SECURITY MANAGER RETURN CODES
      *
       77  ESM  PIC S9(8) COMP VALUE ZERO.
       77  ESM2 PIC S9(8) COMP VALUE ZERO.
      *
      * WORK VARIABLE FOR DAYS LEFT UNTIL PASSWORD EXPIRES
      *
       77  DAYS PIC S9(4) COMP VALUE ZERO.
      *
      * WORK VARIABLE FOR EXEC CICS ADDRESS CWA
      *
       77  CWAPTR POINTER.
      *
      * THE INITIAL VALUE OF THE CLIENT NAME FIELD
      *
       77  INITIAL-NAME PIC X(4) VALUE 'A000'.
      *
      * LOG MESSAGE WRITTEN TO CICS CONSOLE FOR SUCCESSFUL LOGONS
      *
       01  MESSAGE-AREA.
           02 MSGTEXT  PIC X(40).
           02 MSGUSER  PIC X(8) VALUE SPACES.
           02 FILLER   PIC X VALUE SPACE.
           02 MSGPW    PIC X(8) VALUE SPACES.
           02 FILLER   PIC X(14) VALUE '  CLIENT NAME '.
           02 MSGCLNT  PIC X(4) VALUE SPACES.
      *
      * LOG MESSAGE WRITTEN TO CICS CONSOLE FOR ERRORS
      *
       01  ERROR-MSG.
           02 FILLER PIC X(40)
                VALUE '*** CICS ERROR ON VERIFY PASSWORD **'.
           02 ERRRESP  PIC S9(8) VALUE ZERO.
           02 FILLER PIC X VALUE SPACE.
           02 ERRRESP2 PIC S9(8) VALUE ZERO.
           02 FILLER PIC X(8) VALUE ' USERID '.
           02 ERRUSER  PIC X(8) VALUE SPACES.
           02 FILLER PIC X VALUE SPACE.
           02 ERRPW    PIC X(8) VALUE SPACES.
           02 FILLER   PIC X(14) VALUE '  CLIENT NAME '.
           02 ERRCLNT  PIC X(4) VALUE SPACES.
      *
       LINKAGE SECTION.
       01  DFHCOMMAREA.
          COPY VERICOMM.
      *
       01  CWA.
           02  LAST-NAME-USED PIC X(4).
           02  FILLER REDEFINES LAST-NAME-USED.
               03  CLNTLTR  PIC X.
               03  CLNTNUMB PIC 999.
      *
       PROCEDURE DIVISION.
      *
      * INITIALIZE THE RETURNED COMMAREA.
      *
           MOVE ZEROES TO RESP
           MOVE ZEROES TO RESP2
           MOVE ZEROES TO DAYSLEFT.
           MOVE ZEROES TO ESMREASON.
           MOVE ZEROES TO ESMRESP.
           MOVE SPACES TO LASTDATE.
           MOVE SPACES TO LASTTIME.
      *
      * Optionally, assign a unique client id in the commarea,
      * if the client does not already have one.
      *
           IF CLNTNAME IS EQUAL TO SPACES OR LOW-VALUES THEN
               PERFORM ASSIGN-CLIENT THRU ASSIGN-CLIENT-EXIT
           END-IF
      *
      *    VERIFY THE USERID AND PASSWORD
      *
           PERFORM VERIFY-PASSWORD THRU VERIFY-PASSWORD-EXIT.
      *
      * END THE PROGRAM
      *
           EXEC CICS RETURN END-EXEC.
           GOBACK.
      *
      * THE FOLLOWING ROUTINE DOES AN EXEC CICS VERIFY PASSWORD
      * TO VERIFY THE USERID AND PASSWORD PASSED IN THE COMMAREA
      * FROM A CICS CLIENT.  THIS ALLOWS A CLIENT ECI PROGRAM
      * TO VALIDATE A USERID AND PASSWORD
      *
       VERIFY-PASSWORD.
      *
      * INITIALZIE THE RETURN CODES
      *
           MOVE ZEROS TO CICSRESP.
           MOVE ZEROS TO CICSRESP2.
           MOVE ZEROS TO ESM.
           MOVE ZEROS TO ESM2.
      *
      * Statement commented out due to lack of support in TS for NT
      *    EXEC CICS VERIFY PASSWORD(CICS-PW) USERID(CICS-USER)
      *         DAYSLEFT(DAYS) ESMREASON(ESM)
      *         ESMRESP(ESM2) LASTUSETIME(LASTUSED) NOHANDLE
      *         RESP(CICSRESP) RESP2(CICSRESP2)
      *    END-EXEC.
      *
      * CAPTURE THE ESM RETURN CODES
      *
           MOVE ESM  TO ESMREASON.
           MOVE ESM2 TO ESMRESP.
      *
           IF CICSRESP IS EQUAL TO DFHRESP(NORMAL) THEN
      * Statement commented out due to lack of support in TS for NT
      *        EXEC CICS FORMATTIME ABSTIME(LASTUSED) DATESEP
      *             DATE(LASTDATE) TIME(LASTTIME) NOHANDLE
      *        END-EXEC
      *
               MOVE DAYS TO DAYSLEFT
      *
               MOVE 'USER ID AND PASSWORD OK' TO MSGTEXT
      *        MOVE CICS-PW TO MSGPW
               MOVE CICS-USER TO MSGUSER
               MOVE CLNTNAME TO MSGCLNT
      * Statement commented out due to lack of support in TS for NT
      *        EXEC CICS WRITE OPERATOR TEXT(MESSAGE-AREA)
      *             TEXTLENGTH(LENGTH OF MESSAGE-AREA)
      *        END-EXEC
      *
           ELSE
      *
      * GET THE CICS ERROR CODES INTO THE COMMAREA
      *
               MOVE CICSRESP  TO RESP
               MOVE CICSRESP2 TO RESP2
      *
      * FORMAT AN ERROR MESSAGE
      *
               MOVE CICSRESP  TO ERRRESP
               MOVE CICSRESP2 TO ERRRESP2
               MOVE CICS-USER TO ERRUSER
               MOVE CICS-PW   TO ERRPW
               MOVE CLNTNAME  TO ERRCLNT
      *
      * WRITE THE ERROR MESSAGE ON THE CICS CONSOLE
      *
               EXEC CICS WRITE OPERATOR TEXT(ERROR-MSG)
                    TEXTLENGTH(LENGTH OF ERROR-MSG)
               END-EXEC
           END-IF.
      *
       VERIFY-PASSWORD-EXIT.
           EXIT.
      *
      * THE FOLLOWING ROUTINE ADDRESSES THE PROBLEM OF HAVING
      * A UNIQUE FOUR CHARACTER CLIENT NAME TO BE ASSIGNED TO EACH
      * CLIENT OR CLIENT APPLICATION.  THIS ROUTINE WILL USE
      * THE FIRST FOUR BYTES IN THE CICS COMMON WORK AREA.  THE
      * CLIENT NAME IS USED IN PLACE OF A CICS TERMINAL ID IN
      * APPLICATIONS WHICH NEED A UNIQUE FOUR CHARACTER NAME.
      * IT IS ONE EXAMPLE OF A SOLUTION TO THIS REQUIREMENT.
      *
       ASSIGN-CLIENT.
      *
      *    GET A POINTER TO THE CWA
      *
           EXEC CICS ADDRESS CWA(CWAPTR) END-EXEC
           SET ADDRESS OF CWA TO CWAPTR
      *
      * CHECK IF THIS IS THE FIRST TIME
      *
           IF CLNTLTR IS LESS THAN 'A' THEN
               MOVE INITIAL-NAME TO LAST-NAME-USED
           ELSE
               IF CLNTNUMB IS EQUAL TO 999 THEN
                   MOVE ZERO TO CLNTNUMB
                   MOVE 'D' TO CLNTLTR
               ELSE
                   ADD 1 TO CLNTNUMB
               END-IF
           END-IF.
      *
      * GET THE NEXT CLIENT NAME INTO THE COMMAREA
      *
           MOVE LAST-NAME-USED TO CLNTNAME.
      *
       ASSIGN-CLIENT-EXIT.
           EXIT.
